#!/bin/bash

# RDF2FS: A Unix File System RDF Store
# Michael Sintek, Gunnar Grimnes
# DFKI GmbH

HERE=`pwd`
ETC=$HERE/etc

# we use Dan Brikcley's FOAF file as example
STORE=danbri
cd $STORE # makes things a lot easier

# some useful scripts we need later
FIRST="awk -f $ETC/first.awk"
SECOND="awk -f $ETC/second.awk"

# Literal-valued properties

# triple queries with one variable:

# s p ?o  ->  cat s/p
echo '** danbri-foaf.rdf#danbri foaf#name ?o'
cat danbri-foaf.rdf#danbri/foaf#name

# ?s p o  ->  grep -l o */p | $FIRST
echo -e '\n** ?s foaf#name "Dan Brickley"'
grep -l "Dan Brickley" */foaf#name | $FIRST

# s ?p o  ->  grep -l o s/* | $SECOND
echo -e '\n** danbri-foaf.rdf#danbri ?p "Dan Brickley"'
grep -l "Dan Brickley" danbri-foaf.rdf#danbri/* | $SECOND

# triple queries with two variables:

# ?s ?p o  ->  grep -l o */*
echo -e '\n** ?s ?p "Dan Brickley"'
grep -l "Dan Brickley" */*

# ?s p ?o  ->  ls -1 */p 
#   plus retrieval of literals (simply with cat)
echo -e '\n** ?s foaf#name ?o'
for f in $(ls -1 */foaf#name ); do
  echo $( echo $f | $FIRST ) \"$( cat $f )\"
done

# s ?p ?o  ->  file pattern s/* plus retrieval of literals (and resources)
echo -e '\n** danbri-foaf.rdf#danbri ?p ?o'
for f in danbri-foaf.rdf#danbri/*; do
  if [ -f $f ]; then
    echo $( echo $f | $SECOND ) \"$( cat $f )\"
  else # resources are just the files (links) in $f
    echo $( echo $f | $SECOND ) $( ls $f )
  fi
done

# (use ^ and $ for exact values/start/end ...)


# Object-valued properties

# triple queries with one variable:
# s p ?o  ->  ls s/p
echo -e '\n** danbri-foaf.rdf#danbri/foaf#knows ?o'
ls danbri-foaf.rdf#danbri/foaf#knows

# s ?p o
# ?s p o

# triple queries with two variables:
# ?s ?p o
# ?s p ?o
# s ?p ?o


# Path expressions:

# s p1 _ p2 ?o  ->  cat s/p1/*/p2
echo -e '\n** path: danbri-foaf.rdf#danbri foaf#knows ?_ foaf#name ?o'
cat danbri-foaf.rdf#danbri/foaf#knows/*/foaf#name

# ?s p1 _ p2 o  ->  grep -l o */p1/*/p2
echo -e '\n** path: ?s foaf#knows ?_ foaf#name "Libby Miller"'
grep -l "Libby Miller" */foaf#knows/*/foaf#name | $FIRST


# Conjunction:

# ?s p1 o1  AND  ?s p2 o2
#    ->
# using intersection (realized with sort and uniq)
# grep -l o1 */p1 | $FIRST | sort -u > tmp1
# grep -l o2 */p2 | $FIRST | sort -u > tmp2
# sort -m tmp1 tmp2 | uniq -d # = intersection

#echo -e '\n** ?s foaf#plan "Save the world" AND ?s uranai#bloodtype "A+"'
echo -e '\n** ?s foaf#plan "Save the world" AND ?s vocab#uranaibloodtype "A+"'

grep -l "Save the world" */foaf#plan | $FIRST | sort -u > tmp1
grep -l "A+" */vocab#uranaibloodtype | $FIRST | sort -u > tmp2
sort -m tmp1 tmp2 | uniq -d
rm tmp1 tmp2

# alternative:
# rough idea: use "or" pattern for second grep (or = {s1,s2,...} in file names)
# S = $( grep -l o1 */p1 | awk -f first | sort -u | 
#        xargs | sed 's/ /,/g' )
# grep -l o2 {$S}/p2
# ...

